from datetime import datetime

from fastapi import (APIRouter, HTTPException, Response,WebSocket, WebSocketDisconnect)
from typing import List
from pydantic import BaseModel
from ...app.services.ai_services import AIService
from ...app.api.network_config import SwitchConfigurator
from ...config import settings
from ..services.network_scanner import NetworkScanner
from ...app.services.traffic_monitor import traffic_monitor
from ...app.models.traffic_models import TrafficRecord
from src.backend.app.api.database import (SessionLocal)
import  asyncio
from fastapi.responses import HTMLResponse
import matplotlib.pyplot as plt
import io
import base64



router = APIRouter(prefix="", tags=["API"])
scanner = NetworkScanner()

# 添加根路径路由
@router.get("/", include_in_schema=False)
async def root():
    return {
        "message": "欢迎使用AI交换机配置系统",
        "docs": f"{settings.API_PREFIX}/docs",
        "redoc": f"{settings.API_PREFIX}/redoc",
        "endpoints": [
            "/parse_command",
            "/apply_config",
            "/scan_network",
            "/list_devices",
            "/batch_apply_config"
        ]
    }

# 添加favicon处理
@router.get("/favicon.ico", include_in_schema=False)
async def favicon():
    return Response(status_code=204)

class BatchConfigRequest(BaseModel):
    config: dict
    switch_ips: List[str]  # 支持多个IP

@router.post("/batch_apply_config")
async def batch_apply_config(request: BatchConfigRequest):
    results = {}
    for ip in request.switch_ips:
        try:
            configurator = SwitchConfigurator()
            results[ip] = await configurator.apply_config(ip, request.config)
        except Exception as e:
            results[ip] = str(e)
    return {"results": results}



@router.get("/test")
async def test_endpoint():
    return {"message": "Hello World"}

@router.get("/scan_network", summary="扫描网络中的交换机")
async def scan_network(subnet: str = "192.168.1.0/24"):
    try:
        devices = scanner.scan_subnet(subnet)
        return {
            "success": True,
            "devices": devices,
            "count": len(devices)
        }
    except Exception as e:
        raise HTTPException(500, f"扫描失败: {str(e)}")

@router.get("/list_devices", summary="列出已发现的交换机")
async def list_devices():
    return {
        "devices": scanner.load_cached_devices()
    }

class CommandRequest(BaseModel):
    command: str

class ConfigRequest(BaseModel):
    config: dict
    switch_ip: str

@router.post("/parse_command", response_model=dict)
async def parse_command(request: CommandRequest):
    """
    解析中文命令并返回JSON配置
    """
    try:
        ai_service = AIService(settings.SILICONFLOW_API_KEY, settings.SILICONFLOW_API_URL)
        config = await ai_service.parse_command(request.command)
        return {"success": True, "config": config}
    except Exception as e:
        raise HTTPException(
            status_code=400,
            detail=f"Failed to parse command: {str(e)}"
        )

@router.post("/apply_config", response_model=dict)
async def apply_config(request: ConfigRequest):
    """
    应用配置到交换机
    """
    try:
        configurator = SwitchConfigurator(
            username=settings.SWITCH_USERNAME,
            password=settings.SWITCH_PASSWORD,
            timeout=settings.SWITCH_TIMEOUT
        )
        result = await configurator.safe_apply(request.switch_ip, request.config)
        return {"success": True, "result": result}
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Failed to apply config: {str(e)}"
        )
@router.get("/traffic/interfaces", summary="获取所有网络接口")
async def get_network_interfaces():
    return {
        "interfaces": traffic_monitor.get_interfaces()
    }

@router.get("/traffic/current", summary="获取当前流量数据")
async def get_current_traffic(interface: str = None):
    return traffic_monitor.get_current_traffic(interface)

@router.get("/traffic/history", summary="获取流量历史数据")
async def get_traffic_history(interface: str = None, limit: int = 100):
    history = traffic_monitor.get_traffic_history(interface)
    return {
        "sent": history["sent"][-limit:],
        "recv": history["recv"][-limit:],
        "time": [t.isoformat() for t in history["time"]][-limit:]
    }

@router.get("/traffic/records", summary="获取流量记录")
async def get_traffic_records(interface: str = None, limit: int = 100):
    with SessionLocal() as session:
        query = session.query(TrafficRecord)
        if interface:
            query = query.filter(TrafficRecord.interface == interface)
        records = query.order_by(TrafficRecord.timestamp.desc()).limit(limit).all()
        return [record.to_dict() for record in records]

@router.websocket("/ws/traffic")
async def websocket_traffic(websocket: WebSocket):
    """实时流量WebSocket"""
    await websocket.accept()
    try:
        while True:
            # 获取所有接口的当前流量
            traffic_data = traffic_monitor.get_current_traffic()
            await websocket.send_json(traffic_data)
            await asyncio.sleep(1)  # 每秒更新一次
    except WebSocketDisconnect:
        print("客户端断开连接")


@router.get("/traffic/plot", response_class=HTMLResponse, summary="流量可视化图表")
async def plot_traffic(interface: str = "eth0", minutes: int = 10):
    # 获取历史数据
    history = traffic_monitor.get_traffic_history(interface)
    time_points = history["time"][-minutes * 60:]
    sent = history["sent"][-minutes * 60:]
    recv = history["recv"][-minutes * 60:]

    # 创建图表
    plt.figure(figsize=(10, 6))
    plt.plot(time_points, sent, label="发送流量 (B/s)")
    plt.plot(time_points, recv, label="接收流量 (B/s)")
    plt.title(f"{interface} 流量监控 - 最近 {minutes} 分钟")
    plt.xlabel("时间")
    plt.ylabel("流量 (字节/秒)")
    plt.legend()
    plt.grid(True)
    plt.xticks(rotation=45)
    plt.tight_layout()

    # 转换为HTML图像
    buf = io.BytesIO()
    plt.savefig(buf, format="png")
    buf.seek(0)
    image_base64 = base64.b64encode(buf.read()).decode("utf-8")
    plt.close()

    return f"""
    <html>
        <head>
            <title>网络流量监控</title>
        </head>
        <body>
            <h1>{interface} 网络流量监控</h1>
            <img src="data:image/png;base64,{image_base64}" alt="流量图表">
            <p>更新时间: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}</p>
        </body>
    </html>
    """